/* eslint-disable no-useless-escape */
/* eslint-disable no-control-regex */
import { dateUtil } from '@csii/vx-util';

const changeDateToText = function(date) {
  const dateType = date.match(/[yMwdhms]/)[0];
  switch (dateType) {
    case 'y':
      return {
        text: '年',
        value: 'year',
      };
    case 'M':
      return {
        text: '月',
        value: 'month',
      };
    case 'w':
      return {
        text: '周',
        value: 'week',
      };
    case 'd':
      return {
        text: '天',
        value: 'day',
      };
    case 'h':
      return {
        text: '小时',
        value: 'hour',
      };
    case 'm':
      return {
        text: '分钟',
        value: 'minute',
      };
    case 's':
      return {
        text: '秒',
        value: 'second',
      };
    default:
      return false;
  }
};

export default {
  // 金额弱校验方法
  weakMoneyRule: function(rule, value, callback) {
    let maxLength = 12;
    if (typeof value === 'object') {
      let { min = 0, max = 0 } = value;
      if (typeof value.max === 'string') {
        let len0 = value.max.split('.')[0].length;
        if (len0 > maxLength) {
          return callback(new Error('最大查询金额为999 999 999 999, 请在0-999 999 999 999范围内查询'));
        }
      }
      if (typeof value.min === 'string') {
        let len1 = value.min.split('.')[0].length;
        if (len1 > maxLength) {
          return callback(new Error('最大查询金额为999 999 999 999, 请在0-999 999 999 999范围内查询'));
        }
      }
      if (parseFloat(min) > parseFloat(max)) {
        return callback(new Error('终止金额应大于起始金额'));
      }
      return callback();
    }
  },
  // 金额弱校验方法
  weakMoneyNewRule: function(rule, value, callback) {
    let maxLength = 12;
    if (typeof value === 'object') {
      let { min = 0, max = 0 } = value;
      if (typeof value.max === 'string') {
        let len0 = value.max.split('.')[0].length;
        if (len0 > maxLength) {
          return callback(new Error('最大查询金额为999 999 999 999'));
        }
      }
      if (typeof value.min === 'string') {
        let len1 = value.min.split('.')[0].length;
        if (len1 > maxLength) {
          return callback(new Error('最大查询金额为999 999 999 999'));
        }
      }
      if (parseFloat(min) > parseFloat(max)) {
        return callback(new Error('终止金额应大于起始金额'));
      }
      return callback();
    }
  },
  // 金额校验方法
  moneyRule: function(rule, value, callback) {
    const prefix = rule.prefix || '金额';
    if (value === '' || value === null) {
      return callback(new Error(`请输入${prefix}`));
    } else {
      if (value === '0' || value === '0.0' || value === '0.00' || value < 0) {
        return callback(new Error(`${prefix}不允许小于等于0，请重新输入`));
      } else if (value.substr(0, 1) === '.') {
        return callback(new Error(`${prefix}不能够以.开头`));
      } else {
        return callback();
      }
    }
  },
  // 校验数字类型
  checkCountRule: function(rule, value, callback) {
    const prefix = rule.prefix || '此处';
    let regu = /^[1-9]\d*$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}只能输入数字`));
      } else {
        return callback();
      }
    }
  },
  // 利率
  rateCountRule: function(rule, value, callback) {
    const prefix = rule.prefix || '利率';
    let regu = /^[+]{0,1}(\d+)$|^[+]{0,1}(\d+\.\d+)$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}只能输入数字或小数`));
      } else {
        return callback();
      }
    }
  },
  // 校验手机号
  phoneRule: function(rule, value, callback) {
    const prefix = rule.prefix || '手机号码';
    let regu = /^(13[0-9]|14[579]|15[0-3,5-9]|16[6]|17[0135678]|18[0-9]|19[89])\d{8}$/;

    if (value) {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}格式不正确`));
      } else {
        return callback();
      }
    } else {
      return callback();
    }
  },
  // 校验手机号+座机mobile
  mobileRule: function(rule, value, callback) {
    const prefix = rule.prefix || '联系电话';
    let isPhone = /^(13[0-9]|14[579]|15[0-3,5-9]|16[6]|17[0135678]|18[0-9]|19[89])\d{8}$/;
    let isMob = /^0\d{2,3}-?\d{7,8}$/;
    if (value !== '') {
      if (isPhone.test(value) || isMob.test(value)) {
        return callback();
      } else {
        return callback(new Error(`${prefix}格式不正确`));
      }
    } else {
      return callback();
    }
  },
  // 检验邮箱
  // emailRule: function (rule, value, callback) {
  //   const prefix = rule.prefix || '邮箱';
  //   var regu = /^^[a-z0-9A-Z]+[- | a-z0-9A-Z . _]+@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-z]{2,}$/;
  //   if (value !== '') {
  //     if (!regu.test(value)) {
  //       return callback(new Error(`${prefix}格式不正确`));
  //     } else {
  //       return callback();
  //     }
  //   } else {
  //     return callback();
  //   }
  // },
  dateRangeRule: function(rule, value, callback) {
    const dateSpan = rule.dateSpan || null; // 时间跨度
    const dateSpanFix = rule.dateSpanFix; // 时间跨度 - 不符合的提示文字
    const dateRange = rule.dateRange || null; // 可以操作的最长时间
    const dateRangeFix = rule.dateRangeFix; // 可以操作的最长时间 - 不符合的提示文字
    if (value) {
      if (dateSpan) {
        if (!/^(-?\d+)([yMwdhms]s?)$/.test(dateSpan)) {
          console.error('请输入正确的时间跨度, 详情请查看changeDate()方法');
        } else {
          let span = parseInt(dateSpan.match(/-?\d+/g)[0] || 0);
          const spanText = changeDateToText(dateSpan)
            ? changeDateToText(dateSpan)
            : {
                text: '年',
                value: 'year',
              };
          if (span > 0) {
            if (dateUtil.daysBetween(value[1] || value, value[0] || new Date(), spanText.value) > span - 1) {
              if (dateSpanFix) {
                return callback(new Error(dateSpanFix));
              }
              return callback(new Error(`单次查询的时间跨度不得超过${span}个${spanText.text}`));
            }
          } else {
            span = 0 - span;
            if (dateUtil.daysBetween(value[1] || new Date(), value[0] || value, spanText.value) > span - 1) {
              if (dateSpanFix) {
                return callback(new Error(dateSpanFix));
              }
              return callback(new Error(`单次查询的时间跨度不得超过${span}个${spanText.text}`));
            }
          }
        }
      }
      if (dateRange) {
        if (!/^(-?\d+)([yMwdhms]s?)$/.test(dateRange)) {
          console.error('请输入正确的可操作时间长度, 详情请查看changeDate()方法');
        } else {
          let range = parseInt(dateRange.match(/-?\d+/g)[0] || 0);
          const rangeText = changeDateToText(dateRange)
            ? changeDateToText(dateRange)
            : {
                text: '年',
                value: 'year',
              };
          if (range < 0) {
            range = 0 - range;
            if (dateUtil.daysBetween(new Date(), value[0], rangeText.value) > range - 1) {
              if (dateRangeFix) {
                return callback(new Error(dateRangeFix));
              }
              return callback(new Error(`系统可提供近${range}${rangeText.text}内的结果查询`));
            }
          } else {
            if (dateUtil.daysBetween(value[1], new Date(), rangeText.value) > range - 1) {
              if (dateRangeFix) {
                return callback(new Error(dateRangeFix));
              }
              return callback(new Error(`系统可提供近${range}${rangeText.text}内的结果查询`));
            }
          }
        }
      }
    }
    return callback();
  },
  /**
   * @description 金额范围检测
   * @param {*} rule
   * @param {*} value
   * @param {*} callback
   */
  moneyRangeRule: function(rule, value, callback) {
    const prefix = rule.prefix || '金额';
    let min = value.min;
    let max = value.max;
    // if(min.toString().length > 12) {
    //   return callback(new Error('您输入的金额超出范围!'));
    // }
    if (min && !max) {
      return callback(new Error(`终止${prefix}不能为空`));
    }
    if (!min && max) {
      return callback(new Error(`起始${prefix}不能为空`));
    }
    // TODO 待优化
    if (parseFloat(min) < 0 || parseFloat(max) > 99999999) {
      return callback(new Error(`您输入的${prefix}超出范围`));
    }
    if (parseFloat(max) < parseFloat(min)) {
      return callback(new Error(`您输入的${prefix}范围有误`));
    }
    return callback();
  },

  // 校验身份证号
  idCardRule: function(rule, value, callback) {
    const prefix = rule.prefix || '身份证号码';
    let regu = /^[1-9]\d{7}((0\d)|(1[0-2]))(([0|1|2]\d)|3[0-1])\d{3}$|^[1-9]\d{5}[1-9]\d{3}((0\d)|(1[0-2]))(([0|1|2]\d)|3[0-1])\d{3}([0-9]|X)$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`您输入的${prefix}格式不正确`));
      } else {
        return callback();
      }
    }
  },

  // 校验email
  emailRule: function(rule, value, callback) {
    const prefix = rule.prefix || 'e-mail';
    // const reg = /^[A-Za-z0-9\u4e00-\u9fa5]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$/;
    const reg = /[\w!#$%&'*+/=?^_`{|}~-]+(?:\.[\w!#$%&'*+/=?^_`{|}~-]+)*@(?:[\w](?:[\w-]*[\w])?\.)+[\w](?:[\w-]*[\w])?/;
    if (value !== '') {
      if (reg.test(value)) {
        callback();
      } else {
        return callback(new Error(`请输入正确的${prefix}`));
      }
    } else {
      callback();
    }
  },
  // 金额不能超过100万校验
  maxNumRule: function(rule, value, callback) {
    const prefix = rule.prefix || '金额';
    let maxNum = Number('1000000');
    if (Number(value) > maxNum) {
      callback(new Error(`输入${prefix}不能大于100万`));
    } else {
      callback();
    }
  },
  // 票据号码不能输入中文及特殊字符
  billNoRule: function(rule, value, callback) {
    const ruleReg = /[`~!@#$%^&*()_\+=<>?:"{}|,.\/:'\\[\].~！@#￥&*（）——、+={}|《》？：“”【】、；‘’，。、]/gi;
    const ruleCNReg = /[\u4e00-\u9fa5]+/g;
    if (ruleReg.test(value) || ruleCNReg.test(value)) {
      return callback(new Error(`${rule.prefix || '票据号码'}格式错误`));
    } else {
      callback();
    }
  },

  // 收款方名称不能输入特殊字符
  payeeAcNameRule: function(rule, value, callback) {
    let iconRule = /[`~!@#$%^&*()_\+=<>?:"{}|,.\/:'\\[\].~！@#￥&*（）——、+={}|《》？：“”【】、；‘’，。、]/gi;
    if (iconRule.test(value)) {
      return callback(new Error(`${rule.prefix || '收款方名称'}不能输入特殊字符`));
    } else {
      callback();
    }
  },
  // 用途校验
  purposeRule: function(rule, value, callback) {
    const prefix = rule.prefix || '用途';
    if (value === '') {
      callback();
    } else {
      // eslint-disable-next-line no-unused-vars
      let len = 0;
      for (let i = 0; i < value.length; i++) {
        // 取出单个字符
        let c = value.charAt(i);
        if (c.match(/[^\x00-\xff]/gi) !== null) {
          len += 2;
        } else {
          return callback(new Error(`请输入${prefix}`));
        }
      }
    }
  },
  // 校验账号
  accountRule: function(rule, value, callback) {
    const prefix = rule.prefix || '账户';
    let regu = /^[a-zA-Z0-9\-]{16,32}$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`您输入的${prefix}格式不正确`));
      } else {
        return callback();
      }
    } else {
      return callback(new Error(`请输入${prefix}`));
    }
  },
  // 校验6位数字
  verCodeRule: function(rule, value, callback) {
    const prefix = rule.prefix || '验证码';
    let regu = /^\d{6}$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}错误，请重新输入`));
      } else {
        return callback();
      }
    } else {
      return callback(new Error('请输入6位数字'));
    }
  },
  // 校验姓名
  nameRule: function(rule, value, callback) {
    const prefix = rule.prefix || '姓名';
    let regu = /^[\u4E00-\u9FA5]{2,6}$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`您输入的${prefix}格式不正确`));
      } else {
        return callback();
      }
    } else {
      return callback(new Error(`请输入${prefix}`));
    }
  },
  // 校验日期半年范围
  dateRangeHalfYear: function(rule, value, callback) {
    const prefix = rule.prefix || '日期';
    if (value.length < 2) return callback(new Error(`请输入正确的${prefix}`));
    if (dateUtil.daysBetween(new Date(), value[0], 'year') > 2) {
      return callback(new Error('系统可提供近3年内的结果查询'));
    }
    if (dateUtil.daysBetween(value[1], new Date()) > 0) {
      return callback(new Error('日期范围不合法'));
    }
    const start = dateUtil.changeDate('-6M', value[1]);
    if (dateUtil.daysBetween(start, value[0]) > 0) {
      return callback(new Error('单次查询的时间跨度不得超过6个月'));
    }
    return callback();
  },
  // 校验邮政编码
  postCodeRule: function(rule, value, callback) {
    const prefix = rule.prefix || '邮政编码';
    let regu = /^[0-9]{6}$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`您输入的${prefix}不正确`));
      } else {
        return callback();
      }
    } else {
      return callback(new Error(`请输入${prefix}`));
    }
  },
  // 校验别名（0-5）
  aliasRule: function(rule, value, callback) {
    let regu = /^[\u4E00-\u9FA5]{0,5}$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`您输入的${rule.prefix || '别名'}不正确`));
      } else {
        return callback();
      }
    } else {
      return callback(new Error(`请输入${rule.prefix || '别名'}`));
    }
  },
  // 日期范围三个月校验
  dateRangeThreeMonthRule: function(rule, value, callback) {
    const prefix = rule.prefix || '单次查询';
    if (value !== '') {
      const start = dateUtil.changeDate('-3M', value[1]);
      if (dateUtil.daysBetween(start, value[0]) > 0) {
        return callback(new Error(`${prefix}的时间跨度不得超过3个月`));
      }
      return callback();
    }
  },
  validateacceptorName: function(rule, value, callback) {
    if (value === '' || value === null) {
      return callback(new Error('承兑人名称不能为空'));
    } else if ((value.indexOf('银行') !== -1) | (value.indexOf('储蓄') !== -1) | (value.indexOf('信用社') !== -1) | (value.indexOf('财务有限公司') !== -1)) {
      return callback(new Error('不能包含“银行、储蓄、信用社、财务有限公司”字段'));
    } else {
      callback();
    }
  },
  // 校验日期两年范围
  dateRangeTwoYearsRule: function(rule, value, callback) {
    const prefix = rule.prefix || '日期';
    if (value.length < 2) return callback(new Error(`请输入正确的${prefix}`));
    if (dateUtil.daysBetween(new Date(), value[0], 'year') > 1) {
      return callback(new Error('系统可提供近2年内的结果查询'));
    }
    if (dateUtil.daysBetween(value[1], new Date()) > 0) {
      return callback(new Error(`${prefix}范围不合法`));
    }
    const start = dateUtil.changeDate('-3M', value[1]);
    if (dateUtil.daysBetween(start, value[0]) > 0) {
      return callback(new Error('单次查询的时间跨度不得超过3个月'));
    }
    return callback();
  },
  //
  numRangeRule: function(rules, value, callback) {
    const { max = 0, prefix = '' } = rules;
    if (!value) {
      callback();
    } else {
      if (value > Number(max)) {
        callback(new Error(prefix));
      }
      const reg = /^(([1-9][0-9]*)|(([0].\d{0,2}|[1-9][0-9]*.\d{0,2})))$/;
      if (!reg.test(value)) {
        callback(new Error('只能输入两位小数'));
      }
    }
    callback();
  },
  // 整数
  integerRule: function(rules, value, callback) {
    const { prefix = '' } = rules;
    if (!Number.isInteger(value - 0)) {
      callback(new Error(prefix || '请输入正确的整数'));
    }
    callback();
  },
  // 长度规则
  contentLenRule: function(rules, value, callback) {
    const { prefix = '', min, max } = rules;
    if (value && value.length < min) {
      callback(new Error(`${prefix}长度不得小于${min}位`));
    }
    if (value && value.length > max) {
      callback(new Error(`${prefix}长度不得大于${max}位`));
    }
    callback();
  },
};
